using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Collections;

namespace RayCasting2D {

	public partial class Form1 : Form {

		private const int cursorImageSize = 10;

		private Vector startPosition;
		private Vector endPosition;
		private Scene scene;
		private bool rightMouseButtonPressed = false;

		public Form1() {
			InitializeComponent();

			DoubleBuffered = true;
			KeyPreview = true;

			Paint += new PaintEventHandler(Form1_Paint);
			MouseDown += new MouseEventHandler(Form1_MouseDown);
			MouseUp += new MouseEventHandler(Form1_MouseUp);

			startPosition = new Vector(150, 150);

			LoadScene("RayCasting Test.scn.xml");
		}

		void Form1_MouseUp(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Right) {
				rightMouseButtonPressed = false;
				Cursor.Position = new Point(Left + (int)endPosition.X, Top + (int)endPosition.Y);
			}
		}

		void Form1_MouseDown(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Right) {
				rightMouseButtonPressed = true;
				Cursor.Position = new Point(Left + (int)startPosition.X, Top + (int)startPosition.Y);
			}
		}

		void Form1_Paint(object sender, PaintEventArgs e) {
			scene.Draw(e.Graphics);

			if (rightMouseButtonPressed) {
				startPosition = Vector.FromPoint(PointToClient(Cursor.Position));
			} else {
				endPosition = Vector.FromPoint(PointToClient(Cursor.Position));
			}

			e.Graphics.FillEllipse(new SolidBrush(Color.Blue), new Rectangle((int)startPosition.X - cursorImageSize / 2, (int)startPosition.Y - cursorImageSize / 2, cursorImageSize, cursorImageSize));
			
			Vector direction = endPosition - startPosition;
			float length = direction.Magnitude;
			direction.Normalize();
			Scene.RayCastingResult r = scene.RayCast(startPosition, direction, length);

			Point drawnEndPosition = new Point();

			if (r.DoCollide) {
				drawnEndPosition = r.Position;
			} else {
				drawnEndPosition = endPosition;
			}

			e.Graphics.DrawLine(new Pen(Color.Red), startPosition, drawnEndPosition);
			e.Graphics.FillEllipse(new SolidBrush(Color.Red), new Rectangle((int)drawnEndPosition.X - cursorImageSize / 2, (int)drawnEndPosition.Y - cursorImageSize / 2, cursorImageSize, cursorImageSize));
			
			e.Graphics.FillRectangle(new SolidBrush(Color.White), new Rectangle(0, 0, Width, 20));
			e.Graphics.DrawString("Click and hold the right mouse button to drag the blue dot", Font, new SolidBrush(Color.Black), new PointF(0,3));
			Invalidate();
		}

		public void LoadScene(string filePath) {
			XmlTextReader reader = new XmlTextReader(filePath);

			scene = null;
			Bitmap tileSetBitmap = null;
			int tileSetHTileCount = 0;
			int tileSetVTileCount = 0;
			int tileSize = 0;

			while (reader.Read()) {
				if (reader.NodeType != XmlNodeType.Element) continue;

				switch (reader.Name) {

					case "Scene":

						tileSize = int.Parse(reader.GetAttribute("TileSize"));
						scene = new Scene(int.Parse(reader.GetAttribute("Width")), int.Parse(reader.GetAttribute("Height")), tileSize);
						break;

					case "Dependencies":

						tileSetBitmap = new Bitmap(reader.GetAttribute("TileSheetFile"));
						tileSetHTileCount = tileSetBitmap.Width / tileSize;
						tileSetVTileCount = tileSetBitmap.Height / tileSize;
						break;

					case "Tile":

						Tile tile = new Tile();

						int tileIndex = int.Parse(reader.GetAttribute("Index"));
						int x = int.Parse(reader.GetAttribute("X"));
						int y = int.Parse(reader.GetAttribute("Y"));
						bool hFlip = reader.GetAttribute("HFlip") == "1";
						bool vFlip = reader.GetAttribute("VFlip") == "1";

						int tileY = (int)Math.Floor((double)tileIndex / tileSetHTileCount);
						int tileX = tileIndex % tileSetHTileCount;

						Rectangle sourceRect = new Rectangle(tileSize * tileX, tileSize * tileY, tileSize, tileSize);
						Bitmap tileBitmap = new Bitmap(tileSize, tileSize);
						Graphics tileGraphics = Graphics.FromImage(tileBitmap);
						tileGraphics.DrawImage(tileSetBitmap, new Rectangle(0, 0, tileSize, tileSize), sourceRect, GraphicsUnit.Pixel);

						tile.Bitmap = tileBitmap;
						tile.HorizontalFlip = hFlip;
						tile.VerticalFlip = vFlip;

						tile.BuildSolidityMap();

						scene.SetTile(x, y, tile);						
						break;

				}

			}

			reader.Close();

			scene.Draw(this.CreateGraphics());
		}

	}
}