using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Collections;

namespace RayCasting2D {

	public class Tile {

		private Bitmap bitmap;
		private bool horizontalFlip;
		private bool verticalFlip;
		private bool[,] solidityMap;

		public void BuildSolidityMap() {
			solidityMap = new bool[bitmap.Width, bitmap.Height];

			for (int x = 0; x < bitmap.Width; x++) {
				for (int y = 0; y < bitmap.Height; y++) {
					// if the pixel is not transparent set the solidity map value to "true"
					solidityMap[x, y] = bitmap.GetPixel(x, y).A != 0;
				}
			}

			if (horizontalFlip) {
				bool[,] newMap = new bool[bitmap.Width, bitmap.Height];
				int transformedMapX = bitmap.Width - 1;
				for (int x = 0; x < bitmap.Width; x++) {
					for (int y = 0; y < bitmap.Height; y++) {
						newMap[x, y] = solidityMap[transformedMapX, y];
					}
					transformedMapX--;
				}

				solidityMap = newMap;
			}

			if (verticalFlip) {
				bool[,] newMap = new bool[bitmap.Width, bitmap.Height];
				int transformedMapY = bitmap.Height - 1;
				for (int y = 0; y < bitmap.Height; y++) {
					for (int x = 0; x < bitmap.Width; x++) {
						newMap[x, y] = solidityMap[x, transformedMapY];
					}
					transformedMapY--;
				}

				solidityMap = newMap;
			}
		}

		public bool[,] SolidityMap {
			get { return solidityMap; }
		}
		
		public bool VerticalFlip {
			get { return verticalFlip; }
			set { verticalFlip = value; }
		}
		
		public bool HorizontalFlip {
			get { return horizontalFlip; }
			set { horizontalFlip = value; }
		}
		
		public Bitmap Bitmap {
			get { return bitmap; }
			set { bitmap = value; }
		}

	}

}
